*Examining Publication Bias – A Simulation-Based Evaluation of Statistical Tests on Publication Bias
*Andreas Schneck

*last checked 14.09.2017

set dp period
cap log close
set matsize 1000
set scheme burd

log using simul, replace

*power detecting effect for studies in MA
power onecorrelation 0 0.05, n(164) // small effect small n
scalar lnle =  r(power) 
power onecorrelation 0 0.1, n(164) // medium effect small n
scalar lnme =  r(power) 
power onecorrelation 0 0.15, n(164) // medium effect small n
scalar lnhe =  r(power) 
power onecorrelation 0 0.05, n(500) // small effect small n
scalar hnle =  r(power) 
power onecorrelation 0 0.1, n(500) // medium effect small n
scalar hnme =  r(power) 
power onecorrelation 0 0.15, n(500) // medium effect small n
scalar hnhe =  r(power)

*average power   
di (lnle+lnme+lnhe+hnle+hnme+hnhe) /6

*load simulation results
clear
use "sim1.dta", clear
append using "sim2.dta", gen(run)
append using "sim3.dta", gen(run2)
append using "sim4.dta", gen(run3)
replace run = run2+1 if run==. & run2!=.
replace run = run3+2 if run==. & run2==. & run3!=.
replace run = run +1
fre run

drop run2 run3
mvdecode _all, mv(-999=.) 

*Labeling
label var reg_count "regressions executed per MA"
label var r_time "run-time MA in sec."
label var k "number of trials in MA"
label var pb "intent to commit PB"
label var trim "intend to commit trimming (p-hacking) instead of file-drawer"
label define trim 0"file-drawer" 1"p-hacking"
label value trim trim
label var pb_int "actual share of intended PB" 
label var pb_com "actual share of committed PB"
label var pb_suc "actual share of successful PB"
label var het "degree of effect heterogeneity"
label var mu_n "sim. number of observation in trials"
label var mean_N "actual number of observation in trials"
label var es "effect size of trials"
label var I_sq "degree of effect heterogeneity in I²"
label var pu_p "p-uniform: p-value"
label var pu_N "p-uniform: number of included trials"
label var pet_t "FAT/PET: t-value of PET"
label var fat_t "FAT/PET: p-value of FAT"
label var ct3_p "caliper test 3%: p-value"
label var ct3_N "caliper test 3%: number of included trials"
label var ct3_P "caliper test 3%: proportion over-caliper"
label var ct5_p "caliper test 5%: p-value"
label var ct5_N "caliper test 5%: number of included trials"
label var ct5_P "caliper test 5%: proportion over-caliper"
label var ct10_p "caliper test 10%: p-value"
label var ct10_N "caliper test 10%: number of included trials"
label var ct10_P "caliper test 10%: proportion over-caliper"
label var ct15_p "caliper test 15%: p-value"
label var ct15_N "caliper test 15%: number of included trials"
label var ct15_P "caliper test 15%: proportion over-caliper"
label var tes_p "TES: p-value"
label var tes_pE "TES: number of expected significant results"
label var tes_pO "TES: number of observed significant results"
label var run "run of simulation (1-4)"

replace het = het/100
*gen es_n = es
replace es = 2 if es ==1
replace es = 1 if es == 0.5
replace es = 3 if es == 1.5
replace es = 4 if het ==1
label define es 0"es = 0" 1"es = 0.5" 2"es = 1" 3"es = 1.5" 4"es = het [0;2]"
label value es es

replace I_sq = 0 if I_sq<0

order run, after(r_time)
order het, after(trim)
order mu_n, after(trim)
order es, after(mu_n)
order mean_N, before(pb_int)
order I_sq, before(pb_int)

*Codebook refers to here
*Number of regressions
sum reg_count, detail
scalar rc = r(sum)
di rc

*number of total published primary studies
sum k
di r(sum)

************************
************************
*Dataset used for analysis
save basedata.dta, replace

*labeling pb_tests as either significant or not (p<0.05)
gen fat_p = ttail(k, fat_t)

*Degree of heterogeneity conditional on n
bysort mu_n: sum I_sq if pb==0 & het == 1

foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
gen `var'_sig = .
replace `var'_sig=0 if `var'>=0.05 & `var'!=.
replace `var'_sig=1 if `var'<0.05
}

gen pb_p = pb_suc/pb_com
gen pb_c = pb_com/k
gen pb_s =  pb_suc/k

*Impact of intend to commit publication bias on actual/successful publication bias as well as its impact on the meta-analytic p-value
gen pet_p = ttail(k, fat_t)

gen p_inf = .

matrix res_pb = J(80,3,.)
local rcount = 1
local rowname = ""
local colname = ""
forvalues tr = 0(1)1 {
forvalues pb = 50(50)100 {
forvalues n = 100(400)500 {
forvalues k = 100(900)1000 {
forvalues es = 0(1)4 {

local temp = "tr`tr'pb`pb'n`n'k`k'es`es'"
local rowname = "`rowname' `temp'"
local ccount = 1

sum pet_p  if es == `es' & mu_n ==`n' & k==`k' & pb ==0
local p_ub = r(mean)
sum pet_p  if es == `es' & mu_n ==`n' & k==`k' & trim ==`tr' & pb ==`pb'
local p_b = r(mean)
di `p_b'
di `p_ub'
replace p_inf = `p_b'/`p_ub' if es == `es' & mu_n ==`n' & k==`k' & trim ==`tr' & pb ==`pb'

local ccount = 1
foreach var of varlist pb_c pb_p p_inf {
sum `var' if trim==`tr' & pb==`pb' & mu_n==`n' & k==`k' & es==`es'
matrix res_pb[`rcount',`ccount'] = r(mean)
local ccount = `ccount'+1

if `rcount'==1 {
local ctemp = "`var'" 
local colname = "`colname' `ctemp'"
}
}
local rcount = `rcount'+1


}
}
}
}
}

matrix rownames res_pb =`rowname'
matrix colnames res_pb =`colname'

putexcel set results, sheet(pb, replace) modify
putexcel A1=matrix(res_pb),  names

*rescaling and mean centering of pb_suc/ com
sum pb_c if pb!=0
sum pb_s if pb!=0

gen pb_com_c = pb_c*10
gen pb_suc_c = pb_s*10
replace pb_suc_c = 0 if pb_com==0

sum pb_com_c pb_suc_c if pb!=0
sum pb_com_c if pb !=0, detail
replace pb_com_c = (pb_com_c-r(mean))
sum pb_suc_c if pb !=0, detail
replace pb_suc_c = (pb_suc_c-r(mean))

*Regression results of pb conditions
reg pb_c i.mu_n i.es if pb==50
outreg2 using pb_res1.doc, replace se dec(3) alpha(0.001, 0.01, 0.05)

reg pb_c i.mu_n i.es if pb==100
outreg2 using pb_res1.doc, append se dec(3) alpha(0.001, 0.01, 0.05)

reg pb_p i.mu_n i.es i.trim if pb!=0
outreg2 using pb_res1.doc, append se dec(3) alpha(0.001, 0.01, 0.05)

reg p_inf pb_com_c pb_suc_c i.trim if pb!=0
outreg2 using pb_res1.doc, append se dec(3) alpha(0.001, 0.01, 0.05)

*rescale I² for regression results
replace I_sq = I_sq*10

************************
*regression results
*power
capture erase p_res.doc
foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
reg `var'_sig i.k I_sq i.trim pb_com_c pb_suc_c if pb!=0
test _b[_cons] = 0.8
outreg2 using p_res.doc, append se dec(3) alpha(0.001, 0.01, 0.05)
}
*false positive rate
capture erase a_res.doc
foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
reg `var'_sig i.k I_sq if pb==0
test _b[_cons] = 0.05
outreg2 using a_res.doc, append se dec(3) alpha(0.001, 0.01, 0.05)
}

************************
*results by condition
*power
matrix res_power = J(80,14,.)
local rcount = 1
local rowname = ""
local colname = ""
forvalues tr = 0(1)1 {
forvalues pb = 50(50)100 {
forvalues n = 100(400)500 {
forvalues k = 100(900)1000 {
forvalues es = 0(1)4 {
local temp = "tr`tr'pb`pb'n`n'k`k'es`es'"
local rowname = "`rowname' `temp'"
local ccount = 1
foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
sum `var'_sig if trim==`tr' & pb==`pb' & mu_n==`n' & k==`k' & es==`es'
matrix res_power[`rcount',`ccount'] = r(mean)
local ccount = `ccount'+1
if `rcount'==1 {
local ctemp = "`var'_s `var'_p" 
local colname = "`colname' `ctemp'"
}
bitest `var'_sig == 0.8 if trim==`tr' & pb==`pb' & mu_n==`n' & k==`k' & es==`es'
matrix res_power[`rcount',`ccount'] = r(p)
local ccount =  `ccount'+1

} 
local rcount = `rcount'+1
}
}
}
}
}

matrix rownames res_power =`rowname'
matrix colnames res_power =`colname'

putexcel set results, sheet(power, replace) modify
putexcel A1=matrix(res_power),  names

*false positive rate
matrix res_alpha = J(20,14,.)
local rcount = 1
local rowname = ""
local colname = ""

forvalues n = 100(400)500 {
forvalues k = 100(900)1000 {
forvalues es = 0(1)4 {
local temp = "n`n'k`k'es`es'"
local rowname = "`rowname' `temp'"

local ccount = 1
foreach var of varlist pu_p fat_p tes_p ct3_p ct5_p ct10_p ct15_p {
sum `var'_sig if  pb==0 & mu_n==`n' & k==`k' & es==`es'
matrix res_alpha[`rcount',`ccount'] = r(mean)
local ccount =  `ccount'+1

if `rcount'==1 {
local ctemp = "`var'_s `var'_p" 
local colname = "`colname' `ctemp'"
}

bitest `var'_sig == 0.05 if  pb==0 & mu_n==`n' & k==`k' & es==`es'
matrix res_alpha[`rcount',`ccount'] = r(p)
local ccount =  `ccount'+1
} 
local rcount = `rcount'+1
}
}
}
di "`rowname'"
matrix rownames res_alpha =`rowname'
matrix colnames res_alpha =`colname'
putexcel set results, sheet(alpha, replace) modify
putexcel A1=matrix(res_alpha),  names
*************************************
*************************************
log close
