*Examining Publication Bias – A Simulation-Based Evaluation of Statistical Tests on Publication Bias
*Andreas Schneck

*last checked 19.06.2017

clear
set matsize 10000
macro drop _all

scalar run = 1
capture matrix drop A

*********************
*DGP-SETUP
*********************

*foreach trim of numlist 1 {
forvalues trim = 0(1)1 {
scalar trim = `trim'

*Publication Bias
forvalues pb = 0(50)100 {
scalar pb = `pb'

*Effekt eig. 2
forvalues effect = 0(0.5)1.5 {
local es = `effect'

*Heterogeneity
forvalues het = 0(100)100 {
local het = `het'

*Casenumber
forvalues n = 100(400)500 {
local n = `n'

*Casenumber
/*forvalues n = 500 {
foreach n of numlist 500 {
global n = `n'
*/

*Study number
*forvalues k =  1000 {
foreach k of numlist 100(900)1000 {

local k = `k'

if run == 1 {	//  first iteration
*,`n'
matrix A = (`trim',`pb',`es',`het',`k', `n')
scalar run = run +1
}
else  {			// all others
*`n'
matrix AE = (`trim',`pb',`es',`het',`k', `n')
matrix A = (A\AE)
scalar run = run +1
}

}
}
}
}
}
}
*}

*n 
matrix colnames A = trim pb es het k n //design-matrix 

svmat A, names(col)


*exclude unplausible/ redundant combinations
drop if het==100 & es!=0	//different baseline effect-sizes if heterogeneity 100%
drop if trim==1 & pb==0		//different PB-Modes if PB 0%

sum pb

*gen pb_d = 0 if pb==0
*replace pb_d = 1 if pb != 0
*sum pb_d
*local pb = r(N)*(1-r(mean))
*local npb = r(N)*(r(mean))
*capture drop d_pb

*********************
*Power-analysis (one sample wald-test)
*********************
sum trim if pb==0
scalar an = `r(N)'

sum trim if pb!=0
scalar bn = `r(N)'

power oneproportion 0.06 0.05 , power(0.8) test(wald)
local apn =  `r(N)'
di `apn'
power oneproportion 0.8 0.77, power(0.8) test(wald)
local bpn =  `r(N)'
di `bpn'

*Graphics for possible Ns
power oneproportion 0.06 0.05, n(400(200)5000) graph(xline(`apn')) test(wald)
graph export alpha_power.wmf, replace
power oneproportion 0.8 0.77, n(400(200)5000) graph(xline(`bpn')) test(wald)
graph export beta_power.wmf, replace

di `apn' +`bpn'

di "alpha multiplier " `apn'
di "power multiplier " `bpn'

scalar eapn = round(`apn'/4,1)	//alpha: increase casenumber by proportional need 
scalar ebpn = round(`bpn'/4,1) //beta: increase casenumber by proportional need 

di eapn
di ebpn

*********************
*Design expansions
*********************

*Design Matrix every case one line
save sim_design_matrix.dta, replace

*Design runtime matrix - small scaled version of full-design
scalar aexp = round(eapn/ebpn, 1)
expand aexp if pb==0
save runtime_design.dta, replace

*full-design
scalar aexp = round(eapn/ebpn, 1)
use sim_design_matrix.dta,clear
expand ebpn if pb!=0
expand eapn if pb==0


save sim_design.dta, replace

*11 minutes runtime p. runtime matrix
*total runtime estimate
*3.5 if 2core 45
*5.15 if 4 core 33
*2.79 if 1 core 72.1
di (ebpn*5)/60
