*Examining Publication Bias – A Simulation-Based Evaluation of Statistical Tests on Publication Bias
*Andreas Schneck

*last checked 19.06.2017


*Simmulation power/alpha PB-Tests

timer clear
clear
capture matrix drop res
scalar reg_count = 0
macro drop _all
matrix drop _all
set matsize 1000
set segmentsize 50m
set niceness 5
***
***
cd "C:\Users\Andreas Schneck\Desktop\PB_simul\latest"
do fastreg.do
do clear_char.do	
do sim_process.do	// simmulation process (actors behaviour)

*Seeds for different core-processed
***
***
set seed 2105 //1
global run_nr = 1

*set seed 1511 //2
*global run_nr = 2

*set seed 3105 //3
*global run_nr = 3

*set seed 2311 //4
*global run_nr = 4

***
***
*Design Matrix
***
***

*cd "C:\Users\Andreas Schneck\Desktop\PB_simul\design"

*different data-sources
*use runtime_design.dta, clear		//to compute total runtime estimate
use sim_design.dta, clear			//to assure total functionality
*keep if trim ==0 & pb ==0 & es==0.5 & k==100 & n==100 & het ==0
*use sim_design_matrix.dta, clear	//for final runs


*Set numbers of design rows (in mata because rows may exceed 10000, the stata matrix default)
mata
AD=st_data(., .)
st_local("rowad", strofreal(rows(AD)))
end

qui: describe
local maxline = r(N) 
*************************************
*DGP
*************************************
local trow = 0
global hetrange = 2
global sigma_x = 2
global sigma_e = 10
global sigma_z = 10
global sigma_n = 150
global coefs = 3
global ds = 9
local saveth = 120		// save data after x runs

tuples z1 z2 z3
global ntuples = `ntuples'
forvalues sreg = 1(1)$ntuples {
global tuple`sreg' = "`tuple`sreg''"
}

matrix temp_res = J(1,32,.)


*************************************
*Simmulation-process
*************************************

*cd "C:\Users\Andreas Schneck\Desktop\PB_simul\results"

qui {

forvalues row = 1(1)`rowad' {								//loop through sim-design
local perc = `row'/`rowad'
noisily: di `row' " rows of " `rowad' " finished (`perc'%)"	//info-screan (only line, no time reference
local drop perc
timer clear 1
timer on 1
scalar reg_count = 0		// count regressions 

mata:st_local("trim", strofreal(AD[`row',1]))
mata:st_local("pb", strofreal(AD[`row',2]))
mata:st_local("es", strofreal(AD[`row',3]))
mata:st_local("het", strofreal(AD[`row',4]))
mata:st_local("k", strofreal(AD[`row',5]))
mata:st_local("mu_n", strofreal(AD[`row',6]))

*Simulation-process from simdesign.do
dset, es(`es') pb(`pb') trim(`trim') k(`k') mu_n(`mu_n') het(`het')

************************
************************
*Results Meta-Analyses

clear
svmat regs, names(col)		//load meta-results in stata
*matrix drop regs

gen t = koef/se
gen p = 2*(ttail(df,abs(t)))	//two-sided p-value
gen p_os = ttail(df,t)			// one-sided
local pb = `pb'
sum pb_a						
scalar k = r(N)					//if publication bias tendency
scalar pb_int = r(sum)
di pb_int
sum N
scalar mean_N = r(mean)


*publication bias behaviour/intention
sum pb_c
local pb_com = r(sum)			//if publication bias comitted (opportunity)
di `pb_com'

sum pb_c if t>1.96 & pb_c==1
local pb_suc = r(N)
di `pb_suc'

local het = `het'
local n = `mu_n'
local es = `es'


*p-curve
// van Assen, M. A. L. M., van Aert, R. C. M., & Wicherts, J. M. (2015). Meta-analysis using effect size distributions of only statistically significant studies. Psychological Methods, 20(3), 293-309.
// p.297

gen w = 1/se^2
reg koef [aweight=w]
local p_es = 2 * ttail(e(df_r), _b[_cons]/_se[_cons])
local fe =  _b[_cons]			//fixed effects meta-analysis to compare estimate to


*I2 Heterogeneity Thompson & Higgins 2002
predict res, resid
replace res = w*res^2
sum res
local Isq = (r(sum)-(r(N)-1))/r(sum)

drop w

*preserve
*keep 					// drop all est if p>0.05 t<1.96

sum p if p<0.05		
if r(N)==0 {					//missing if p<0.05 missing	
local pu_N = -999
local pu_p = -999
*restore
}

else {
gen tcv = invttail(df, 0.025) 	//study specific significance threshold

gen CV = tcv * se
gen pmarg = 1-normal((CV-`fe')/se)	//significance threshold in relation to mean effect
gen ph1 = 	1-normal((koef-`fe')/se)	//effect related to mean effect
replace ph1 = 1-ph1 if koef<0
gen q = ph1/pmarg if p<0.05 & koef>=0 //only pos effects (suspected pb) van Aert 2016:727; Simmonsohn 2014:677

replace q = log(q)
sum q
scalar sumq = -r(sum)

local pu_N = r(N)
local pu_p = gammap(r(N), sumq)	//Fishers' method but gamma distribution with k,1 df
scalar drop sumq
drop q
*restore
di `pu_N'
di `pu_p'
}


*TES (with binomial test)

gen power = .
replace power = ttail(df,(invttail(df, 0.025)-(`fe'/(se))))
replace power = power + 1-ttail(df,(invttail(df, 0.975)-(`fe'/(se))))

sum power
*Expected positives
local E = r(sum)

*Observed positives
gen exp = 0
replace exp = 1 if p<0.05
sum exp
local O = r(sum)

local pE = `E'/`k'

*binomial test
bitesti `k' `O' `pE'


local tes_p = r(p_u)
local tes_pE =r(P_p)         
local tes_pO =r(k)/r(N)

drop power exp
local drop O pE

*************************
*FAT_PET
gen prec = 1/se
reg t prec		//FAT

local pet_t = _b[prec]/_se[prec]
local fat_t = _b[_cons]/_se[_cons]

*************************
*Caliper-Test
gen z = abs(invnormal(p_os))
local th = invnormal(0.975)

foreach num of numlist 3 5 10 15 {
local uc = `th' - ((`num'*0.01)*`th')
local oc = `th' + ((`num'*0.01)*`th')
gen cal`num' = 0 if z<=`th' & z > `uc'
replace cal`num' = 1 if z>`th' & z < `oc'

sum cal`num'
if r(N)==0 {
local ct`num'_p = -999
local ct`num'_N = -999
local ct`num'_P = -999
}

else {
bitest cal`num' = 0.5
local ct`num'_p = r(p_u)
local ct`num'_N = r(N)
local ct`num'_P = r(k)/r(N)
}

local drop uc oc th 
drop cal`num'
}


*Compute speed of every single run
timer off 1
timer list 1
local r_time =  r(t1)

*Export results
matrix temp_res[1 ,1] = reg_count
matrix temp_res[1 ,2] = `r_time'
matrix temp_res[1 ,3] = k
matrix temp_res[1 ,4] = `pb'
matrix temp_res[1 ,5] = `trim'
matrix temp_res[1 ,6] = pb_int
matrix temp_res[1 ,7] = `pb_com'
matrix temp_res[1 ,8] = `pb_suc'
matrix temp_res[1 ,9] = `het'
matrix temp_res[1 ,10] = `mu_n'
matrix temp_res[1 ,11] = mean_N
matrix temp_res[1 ,12] = `es'
matrix temp_res[1 ,13] = `Isq'
matrix temp_res[1 ,14] = `pu_p'
matrix temp_res[1 ,15] = `pu_N'
matrix temp_res[1 ,16] = `pet_t'
matrix temp_res[1 ,17] = `fat_t'
matrix temp_res[1 ,18] = `ct3_p'
matrix temp_res[1 ,19] = `ct3_N'
matrix temp_res[1 ,20] = `ct3_P'
matrix temp_res[1 ,21] = `ct5_p'
matrix temp_res[1 ,22] = `ct5_N'
matrix temp_res[1 ,23] = `ct5_P'
matrix temp_res[1 ,24] = `ct10_p'
matrix temp_res[1 ,25] = `ct10_N'
matrix temp_res[1 ,26] = `ct10_P'
matrix temp_res[1 ,27] = `ct15_p'
matrix temp_res[1 ,28] = `ct15_N'
matrix temp_res[1 ,29] = `ct15_P'
matrix temp_res[1 ,30] = `tes_p'
matrix temp_res[1 ,31] = `tes_pE'
matrix temp_res[1 ,32] = `tes_pO'

local drop r_time pb trim pb_com pb_suc het mu_n es Isq pu_p pu_N pet_t fat_t ct3p ///
ct3_N ct3_P ct5_p ct5_N ct5_P ct10_p ct10_N ct10_P ct15_p ct15_N ct15_P tes_p tes_pE tes_pO
scalar drop reg_count k mean_N pb_int
*Store sesult if first run
if `row' == 1 {
mata: E = st_matrix("temp_res")
}


*Store sesult if further run
if `row'> 1 & `trow'!=1 {
mata: E = E\st_matrix("temp_res")
matrix drop temp_res //sinnvoll?
matrix temp_res = J(1,32,.)
}

if `trow' == 1 {
mata: E = st_matrix("temp_res")
matrix drop temp_res //sinnvoll?
matrix temp_res = J(1,32,.)
local trow = 0
}

if `row' == 1 {
local savebit = `saveth'
mata: E = st_matrix("temp_res")
matrix drop temp_res //sinnvoll?
matrix temp_res = J(1,32,.)
}

if `row' == `maxline' {
clear
local last_obs = `maxline'-`saveth'+`savebit'
set obs `last_obs'
mata: newvars = st_addvar("float", ("reg_count", "r_time", "k", "pb", "trim", "pb_int", "pb_com", "pb_suc", "het", "mu_n", "mean_N", "es", "I_sq", "pu_p", "pu_N", "pet_t", "fat_t", "ct3_p", "ct3_N", "ct3_P", "ct5_p", "ct5_N", "ct5_P", "ct10_p", "ct10_N", "ct10_P", "ct15_p", "ct15_N", "ct15_P", "tes_p","tes_pE","tes_pO"))
mata: st_store(., newvars, E)
append using "sim$run_nr.dta"
save "sim$run_nr.dta", replace
clear_char, dataset
mata: mata drop E
}

if `row' == `saveth' {
noisily di "saved"
noisily memory
clear
set obs `savebit'

mata: newvars = st_addvar("float", ("reg_count", "r_time", "k", "pb", "trim", "pb_int", "pb_com", "pb_suc", "het", "mu_n", "mean_N", "es", "I_sq", "pu_p", "pu_N", "pet_t", "fat_t", "ct3_p", "ct3_N", "ct3_P", "ct5_p", "ct5_N", "ct5_P", "ct10_p", "ct10_N", "ct10_P", "ct15_p", "ct15_N", "ct15_P", "tes_p","tes_pE","tes_pO"))
mata: st_store(., newvars, E)

sum r_time

noisily di r(sum)/60

if `row' > `savebit' {
append using "sim$run_nr.dta"
}
save "sim$run_nr.dta", replace
clear_char, dataset
mata: mata drop E
local trow = 1
local saveth = `saveth' + `savebit'
clear
}

}
}


